@@ -1,6 +1,8 @@
 #include "system.h"
 
-#include "rpmbuild.h"
+#include <rpmbuild.h>
+#include <rpmurl.h>
+
 #include "build.h"
 #include "install.h"
 
@@ -57,12 +59,13 @@ static int isSpecFile(const char *specfile)
     int count;
     int checking;
 
-    if (fdFileno((fd = fdOpen(specfile, O_RDONLY, 0))) < 0) {
-	fprintf(stderr, _("Unable to open spec file: %s\n"), specfile);
+    fd = Fopen(specfile, "r.ufdio");
+    if (fd == NULL || Ferror(fd)) {
+	fprintf(stderr, _("Unable to open spec file %s: %s\n"), specfile, Fstrerror(fd));
 	return 0;
     }
-    count = fdRead(fd, buf, sizeof(buf));
-    fdClose(fd);
+    count = Fread(buf, sizeof(buf[0]), sizeof(buf), fd);
+    Fclose(fd);
 
     checking = 1;
     for (s = buf; count--; s++) {
@@ -87,21 +90,27 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	int force, int nodeps)
 {
     int buildAmount = ba->buildAmount;
-    const char *buildRoot = ba->buildRootOverride;
+    const char *buildRootURL = NULL;
     int test = ba->noBuild;
-    FILE *f;
-    const char * specfile;
-    int res = 0;
-    struct stat statbuf;
+    const char * specFile;
+    const char * specURL;
+    int specut;
     char buf[BUFSIZ];
     Spec spec = NULL;
+    int rc;
 
     rpmSetTables(RPM_MACHTABLE_BUILDARCH, RPM_MACHTABLE_BUILDOS);
 
+    if (ba->buildRootOverride)
+	buildRootURL = rpmGenPath(NULL, ba->buildRootOverride, NULL);
+
     if (fromTarball) {
+	FILE *fp;
 	const char *specDir;
 	const char * tmpSpecFile;
 	char * cmd, *s;
+	int res;
+	static const char *zcmds[] = { "cat", "gunzip", "bunzip2", "cat" };
 
 	specDir = rpmGetPath("%{_specdir}", NULL);
 
@@ -110,30 +119,32 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	    tmpSpecFile = rpmGetPath("%{_specdir}/", mktemp(tfn), NULL);
 	}
 
+	isCompressed(arg, &res);
+
 	cmd = alloca(strlen(arg) + 50 + strlen(tmpSpecFile));
-	sprintf(cmd, "gunzip < %s | tar xOvf - Specfile 2>&1 > %s",
-			arg, tmpSpecFile);
-	if (!(f = popen(cmd, "r"))) {
+	sprintf(cmd, "%s < %s | tar xOvf - Specfile 2>&1 > %s",
+			zcmds[res & 0x3], arg, tmpSpecFile);
+	if (!(fp = popen(cmd, "r"))) {
 	    fprintf(stderr, _("Failed to open tar pipe: %s\n"), 
 			strerror(errno));
 	    xfree(specDir);
 	    xfree(tmpSpecFile);
 	    return 1;
 	}
-	if ((!fgets(buf, sizeof(buf) - 1, f)) || !strchr(buf, '/')) {
+	if ((!fgets(buf, sizeof(buf) - 1, fp)) || !strchr(buf, '/')) {
 	    /* Try again */
-	    pclose(f);
+	    pclose(fp);
 
-	    sprintf(cmd, "gunzip < %s | tar xOvf - \\*.spec 2>&1 > %s", arg,
-		    tmpSpecFile);
-	    if (!(f = popen(cmd, "r"))) {
+	    sprintf(cmd, "%s < %s | tar xOvf - \\*.spec 2>&1 > %s",
+		    zcmds[res & 0x3], arg, tmpSpecFile);
+	    if (!(fp = popen(cmd, "r"))) {
 		fprintf(stderr, _("Failed to open tar pipe: %s\n"), 
 			strerror(errno));
 		xfree(specDir);
 		xfree(tmpSpecFile);
 		return 1;
 	    }
-	    if (!fgets(buf, sizeof(buf) - 1, f)) {
+	    if (!fgets(buf, sizeof(buf) - 1, fp)) {
 		/* Give up */
 		fprintf(stderr, _("Failed to read spec file from %s\n"), arg);
 		unlink(tmpSpecFile);
@@ -142,7 +153,7 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	    	return 1;
 	    }
 	}
-	pclose(f);
+	pclose(fp);
 
 	cmd = s = buf;
 	while (*cmd) {
@@ -156,15 +167,16 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	s = cmd + strlen(cmd) - 1;
 	*s = '\0';
 
-	s = alloca(strlen(specDir) + strlen(cmd) + 5);
+	specURL = s = alloca(strlen(specDir) + strlen(cmd) + 5);
 	sprintf(s, "%s/%s", specDir, cmd);
+	res = rename(tmpSpecFile, s);
+	xfree(specDir);
+	xfree(tmpSpecFile);
 	
-	if (rename(tmpSpecFile, s)) {
+	if (res) {
 	    fprintf(stderr, _("Failed to rename %s to %s: %s\n"),
 		    tmpSpecFile, s, strerror(errno));
 	    unlink(tmpSpecFile);
-	    xfree(specDir);
-	    xfree(tmpSpecFile);
 	    return 1;
 	}
 
@@ -183,38 +195,44 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	*cmd = '\0';
 
 	addMacro(NULL, "_sourcedir", NULL, buf, RMIL_TARBALL);
-	xfree(specDir);
-	xfree(tmpSpecFile);
-	specfile = s;
-    } else if (arg[0] == '/') {
-	specfile = arg;
     } else {
+	specURL = arg;
+    }
+
+    specut = urlPath(specURL, &specFile);
+    if (*specFile != '/') {
 	char *s = alloca(BUFSIZ);
 	(void)getcwd(s, BUFSIZ);
 	strcat(s, "/");
 	strcat(s, arg);
-	specfile = s;
+	specURL = s;
     }
 
-    stat(specfile, &statbuf);
-    if (! S_ISREG(statbuf.st_mode)) {
-	fprintf(stderr, _("File is not a regular file: %s\n"), specfile);
-	return 1;
-    }
+    if (specut != URL_IS_DASH) {
+	struct stat st;
+	Stat(specURL, &st);
+	if (! S_ISREG(st.st_mode)) {
+	    fprintf(stderr, _("File is not a regular file: %s\n"), specURL);
+	    rc = 1;
+	    goto exit;
+	}
 
-    /* Try to verify that the file is actually a specfile */
-    if (!isSpecFile(specfile)) {
-	fprintf(stderr, _("File %s does not appear to be a specfile.\n"),
-		specfile);
-	return 1;
+	/* Try to verify that the file is actually a specfile */
+	if (!isSpecFile(specURL)) {
+	    fprintf(stderr, _("File %s does not appear to be a specfile.\n"),
+		specURL);
+	    rc = 1;
+	    goto exit;
+	}
     }
     
     /* Parse the spec file */
 #define	_anyarch(_f)	\
 (((_f)&(RPMBUILD_PREP|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
-    if (parseSpec(&spec, specfile, buildRoot, 0, passPhrase, cookie,
-	_anyarch(buildAmount), force)) {
-	    return 1;
+    if (parseSpec(&spec, specURL, ba->rootdir, buildRootURL, 0, passPhrase,
+		cookie, _anyarch(buildAmount), force)) {
+	rc = 1;
+	goto exit;
     }
 #undef	_anyarch
 
@@ -223,40 +241,48 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 
     /* Check build prerequisites */
     if (!nodeps && checkSpec(spec->sourceHeader)) {
-	freeSpec(spec);
-	return 1;
+	rc = 1;
+	goto exit;
     }
 
     if (buildSpec(spec, buildAmount, test)) {
-	freeSpec(spec);
-	return 1;
+	rc = 1;
+	goto exit;
     }
     
-    if (fromTarball) unlink(specfile);
+    if (fromTarball) Unlink(specURL);
+    rc = 0;
 
-    freeSpec(spec);
-    
-    return res;
+exit:
+    if (spec)
+	freeSpec(spec);
+    if (buildRootURL)
+	xfree(buildRootURL);
+    return rc;
 }
 
-int build(const char *arg, struct rpmBuildArguments *ba, const char *passPhrase,
-	  int fromTarball, char *cookie, const char * rcfile, int force,
-	  int nodeps)
+/** */
+int build(const char * arg, struct rpmBuildArguments * ba,
+	const char * passPhrase, int fromTarball, char * cookie,
+	const char * rcfile, int force, int nodeps)
 {
     char *t, *te;
-    int rc;
+    int rc = 0;
     char *targets = ba->targets;
+#define	buildCleanMask	(RPMBUILD_RMSOURCE|RPMBUILD_RMSPEC)
+    int cleanFlags = ba->buildAmount & buildCleanMask;
 
     if (targets == NULL) {
 	rc =  buildForTarget(arg, ba, passPhrase, fromTarball, cookie,
 		force, nodeps);
-	return rc;
+	goto exit;
     }
 
     /* parse up the build operators */
 
     printf(_("Building target platforms: %s\n"), targets);
 
+    ba->buildAmount &= ~buildCleanMask;
     for (t = targets; *t != '\0'; t = te) {
 	char *target;
 	if ((te = strchr(t, ',')) == NULL)
@@ -264,18 +290,27 @@ int build(const char *arg, struct rpmBuildArguments *ba, const char *passPhrase,
 	target = alloca(te-t+1);
 	strncpy(target, t, (te-t));
 	target[te-t] = '\0';
+	if (*te)
+	    te++;
+	else	/* XXX Perform clean-up after last target build. */
+	    ba->buildAmount |= cleanFlags;
+
 	printf(_("Building for target %s\n"), target);
 
+	/* Read in configuration for target. */
+	rpmFreeMacros(NULL);
 	rpmReadConfigFiles(rcfile, target);
 	rc = buildForTarget(arg, ba, passPhrase, fromTarball, cookie,
 		force, nodeps);
 	if (rc)
-	    return rc;
-
-	freeMacros(NULL);
+	    break;
     }
 
-    return 0;
+exit:
+    /* Restore original configuration. */
+    rpmFreeMacros(NULL);
+    rpmReadConfigFiles(rcfile, NULL);
+    return rc;
 }
 
 #define	POPT_USECATALOG		1000
@@ -294,9 +329,9 @@ extern int noLang;
 static int noBuild = 0;
 static int useCatalog = 0;
 
-static void buildArgCallback(poptContext con, enum poptCallbackReason reason,
-                             const struct poptOption * opt, const char * arg,
-                             const void * data)
+static void buildArgCallback( /*@unused@*/ poptContext con,
+	/*@unused@*/ enum poptCallbackReason reason,
+	const struct poptOption * opt, const char * arg, const void * data)
 {
     struct rpmBuildArguments * rba = (struct rpmBuildArguments *) data;
 
@@ -326,7 +361,7 @@ static void buildArgCallback(poptContext con, enum poptCallbackReason reason,
 	/*@notreached@*/ break;
     case POPT_TARGETPLATFORM:
 	if (rba->targets) {
-	    int len = strlen(rba->targets) + strlen(arg) + 2;
+	    int len = strlen(rba->targets) + 1 + strlen(arg) + 1;
 	    rba->targets = xrealloc(rba->targets, len);
 	    strcat(rba->targets, ",");
 	} else {
@@ -338,6 +373,7 @@ static void buildArgCallback(poptContext con, enum poptCallbackReason reason,
     }
 }
 
+/** */
 struct poptOption rpmBuildPoptTable[] = {
 	{ NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA,
 		buildArgCallback, 0, NULL, NULL },
