@@ -209,10 +209,14 @@ static int getNextHeader(FD_t cfd, /*@out@*/ struct cpioHeader * chPtr)
 /** */
 int cpioFileMapCmp(const void * a, const void * b)
 {
-    const struct cpioFileMapping * first = a;
-    const struct cpioFileMapping * second = b;
+    const char * afn = ((const struct cpioFileMapping *)a)->archivePath;
+    const char * bfn = ((const struct cpioFileMapping *)b)->archivePath;
 
-    return (strcmp(first->archivePath, second->archivePath));
+    /* Match payloads with ./ prefixes as well. */
+    if (afn[0] == '.' && afn[1] == '/')        afn += 2;
+    if (bfn[0] == '.' && bfn[1] == '/')        bfn += 2;
+
+    return strcmp(afn, bfn);
 }
 
 /* This could trash files in the path! I'm not sure that's a good thing */
@@ -326,7 +330,7 @@ static int checkDirectory(const char * filename)
 }
 
 /** */
-static int expandRegular(FD_t cfd, struct cpioHeader * hdr,
+static int expandRegular(FD_t cfd, const struct cpioHeader * hdr,
 			 cpioCallback cb, void * cbData)
 {
     FD_t ofd;
@@ -391,7 +395,7 @@ static int expandRegular(FD_t cfd, struct cpioHeader * hdr,
 }
 
 /** */
-static int expandSymlink(FD_t cfd, struct cpioHeader * hdr)
+static int expandSymlink(FD_t cfd, const struct cpioHeader * hdr)
 {
     char buf[2048], buf2[2048];
     struct stat sb;
@@ -425,7 +429,7 @@ static int expandSymlink(FD_t cfd, struct cpioHeader * hdr)
 }
 
 /** */
-static int expandFifo( /*@unused@*/ FD_t cfd, struct cpioHeader * hdr)
+static int expandFifo( /*@unused@*/ FD_t cfd, const struct cpioHeader * hdr)
 {
     struct stat sb;
 
@@ -443,7 +447,7 @@ static int expandFifo( /*@unused@*/ FD_t cfd, struct cpioHeader * hdr)
 }
 
 /** */
-static int expandDevice( /*@unused@*/ FD_t cfd, struct cpioHeader * hdr)
+static int expandDevice( /*@unused@*/ FD_t cfd, const struct cpioHeader * hdr)
 {
     struct stat sb;
 
