@@ -612,90 +612,6 @@ Header headerRead(FD_t fd, int magicp)
     return h;
 }
 
-int headerGzWrite(FD_t fd, Header h, int magicp)
-{
-    void * p;
-    int length;
-    int_32 l;
-    ssize_t nb;
-
-    p = doHeaderUnload(h, &length);
-
-    if (magicp) {
-	nb = Fwrite(header_magic, sizeof(char), sizeof(header_magic), fd);
-	if (nb != sizeof(header_magic)) {
-	    free(p);
-	    return 1;
-	}
-	l = htonl(0);
-	nb = Fwrite(&l, sizeof(char), sizeof(l), fd);
-	if (nb != sizeof(l)) {
-	    free(p);
-	    return 1;
-	}
-    }
-    
-    nb = Fwrite(p, sizeof(char), length, fd);
-    if (nb != length) {
-	free(p);
-	return 1;
-    }
-
-    free(p);
-    return 0;
-}
-
-Header headerGzRead(FD_t fd, int magicp)
-{
-    int_32 reserved;
-    int_32 * p;
-    int_32 il, dl;
-    int_32 magic;
-    Header h;
-    void * block;
-    int totalSize;
-
-    if (magicp == HEADER_MAGIC_YES) {
-	if (Fread(&magic, sizeof(char), sizeof(magic), fd) != sizeof(magic))
-	    return NULL;
-	if (memcmp(&magic, header_magic, sizeof(magic))) {
-	    return NULL;
-	}
-
-	if (Fread(&reserved, sizeof(char), sizeof(reserved), fd) != sizeof(reserved))
-	    return NULL;
-    }
-    
-    /* First read the index length (count of index entries) */
-    if (Fread(&il, sizeof(char), sizeof(il), fd) != sizeof(il)) 
-	return NULL;
-
-    il = ntohl(il);
-
-    /* Then read the data length (number of bytes) */
-    if (Fread(&dl, sizeof(char), sizeof(dl), fd) != sizeof(dl)) 
-	return NULL;
-
-    dl = ntohl(dl);
-
-    totalSize = sizeof(int_32) + sizeof(int_32) + 
-		(il * sizeof(struct entryInfo)) + dl;
-
-    block = p = xmalloc(totalSize);
-    *p++ = htonl(il);
-    *p++ = htonl(dl);
-
-    totalSize -= sizeof(int_32) + sizeof(int_32);
-    if (Fread(p, sizeof(char), totalSize, fd) != totalSize)
-	return NULL;
-    
-    h = headerLoad(block);
-
-    free(block);
-
-    return h;
-}
-
 /********************************************************************/
 /*                                                                  */
 /* Header dumping                                                   */
