@@ -273,6 +273,42 @@ int rpmInstall(const char * rootdir, const char ** fileArgv, int transFlags,
 			/*@notreached@*/
 		    }
 		}
+		/* If this is a freshen operation, verify an older package is installed */
+		if (interfaceFlags & INSTALL_FRESHEN) {
+		    dbiIndexSet matches;
+		    const char * name;
+		    Header oldH;
+		    int count, freematches;
+		    int j;
+
+		    headerNVR(h, &name, NULL, NULL);
+		
+		    freematches = count = 0;
+		    if (rpmdbFindPackage(db, name, &matches) == 0) {
+			count = dbiIndexSetCount(matches);
+			freematches = 1;
+		    }
+
+		    for (j = 0; j < count; j++) {
+			unsigned int offset = dbiIndexRecordOffset(matches, j);
+			if ((oldH = rpmdbGetRecord(db, offset)) == NULL)
+			    continue;
+			if (rpmVersionCompare(oldH, h) < 0)
+			    continue;
+			/* same or newer package already installed */
+			count = 0;
+			break;
+		    }
+		    if (freematches)
+			dbiFreeIndexRecord(matches);
+
+		    if (count == 0) {
+			headerFree(h);
+			Fclose(fd);
+			break;  /* XXX out of switch */
+		    }
+		    /* Package is newer than those currently installed. */
+		}
 
 		rc = rpmtransAddPackage(rpmdep, h, NULL, fileName,
 			       (interfaceFlags & INSTALL_UPGRADE) != 0,
