@@ -9,140 +9,215 @@
 #include <netinet/in.h>
 #endif	/* __LCLINT__ */
 
-#include "rpmbuild.h"
-
+#include <rpmbuild.h>
+#include <rpmio.h>
 #include <rpmurl.h>
 
-/*@access FD_t@*/
+#ifndef	IPPORT_FTP
+#define	IPPORT_FTP	21
+#endif
+#ifndef	IPPORT_HTTP
+#define	IPPORT_HTTP	80
+#endif
 
-static struct urlstring {
-    const char *leadin;
-    urltype	ret;
-} urlstrings[] = {
-    { "file://",	URL_IS_PATH },
-    { "ftp://",		URL_IS_FTP },
-    { "http://",	URL_IS_HTTP },
-    { "-",		URL_IS_DASH },
-    { NULL,		URL_IS_UNKNOWN }
-};
+/*@access urlinfo@*/
 
-void freeUrlinfo(urlinfo *u)
+#define	URL_IOBUF_SIZE	4096
+int url_iobuf_size = URL_IOBUF_SIZE;
+
+#define	RPMURL_DEBUG_IO		0x40000000
+#define	RPMURL_DEBUG_REFS	0x20000000
+
+int _url_debug = 0;
+#define	DBG(_f, _m, _x)	if ((_url_debug | (_f)) & (_m)) fprintf _x
+
+#define DBGIO(_f, _x)	DBG((_f), RPMURL_DEBUG_IO, _x)
+#define DBGREFS(_f, _x)	DBG((_f), RPMURL_DEBUG_REFS, _x)
+
+/*@only@*/ /*@null@*/ static urlinfo *uCache = NULL;
+static int uCount = 0;
+
+urlinfo XurlLink(urlinfo u, const char *msg, const char *file, unsigned line)
 {
-    if (u->ftpControl >= 0)
-	close(u->ftpControl);
+    URLSANE(u);
+    u->nrefs++;
+DBGREFS(0, (stderr, "--> url %p ++ %d %s at %s:%u\n", u, u->nrefs, msg, file, line));
+    return u;
+}
+
+urlinfo XurlNew(const char *msg, const char *file, unsigned line)
+{
+    urlinfo u;
+    if ((u = xmalloc(sizeof(*u))) == NULL)
+	return NULL;
+    memset(u, 0, sizeof(*u));
+    u->proxyp = -1;
+    u->port = -1;
+    u->urltype = URL_IS_UNKNOWN;
+    u->ctrl = NULL;
+    u->data = NULL;
+    u->bufAlloced = 0;
+    u->buf = NULL;
+    u->httpHasRange = 1;
+    u->httpVersion = 0;
+    u->nrefs = 0;
+    u->magic = URLMAGIC;
+    return XurlLink(u, msg, file, line);
+}
+
+urlinfo XurlFree(urlinfo u, const char *msg, const char *file, unsigned line)
+{
+    URLSANE(u);
+DBGREFS(0, (stderr, "--> url %p -- %d %s at %s:%u\n", u, u->nrefs, msg, file, line));
+    if (--u->nrefs > 0)
+	return u;
+    if (u->ctrl) {
+#ifndef	NOTYET
+	void * fp = fdGetFp(u->ctrl);
+	if (fp) {
+	    fdPush(u->ctrl, fpio, fp, -1);   /* Push fpio onto stack */
+	    Fclose(u->ctrl);
+	} else if (fdio->_fileno(u->ctrl) >= 0)
+	    fdio->close(u->ctrl);
+#else
+	Fclose(u->ctrl);
+#endif
+
+	u->ctrl = fdio->_fdderef(u->ctrl, "persist ctrl (urlFree)", file, line);
+	if (u->ctrl)
+	    fprintf(stderr, _("warning: u %p ctrl %p nrefs != 0 (%s %s)\n"),
+			u, u->ctrl, u->host, u->service);
+    }
+    if (u->data) {
+#ifndef	NOTYET
+	void * fp = fdGetFp(u->data);
+	if (fp) {
+	    fdPush(u->data, fpio, fp, -1);   /* Push fpio onto stack */
+	    Fclose(u->data);
+	} else if (fdio->_fileno(u->data) >= 0)
+	    fdio->close(u->data);
+#else
+	Fclose(u->ctrl);
+#endif
+
+	u->data = fdio->_fdderef(u->data, "persist data (urlFree)", file, line);
+	if (u->data)
+	    fprintf(stderr, _("warning: u %p data %p nrefs != 0 (%s %s)\n"),
+			u, u->data, u->host, u->service);
+    }
+    if (u->buf) {
+	free(u->buf);
+	u->buf = NULL;
+    }
     FREE(u->url);
     FREE(u->service);
     FREE(u->user);
     FREE(u->password);
     FREE(u->host);
     FREE(u->portstr);
-    FREE(u->path);
     FREE(u->proxyu);
     FREE(u->proxyh);
 
-    FREE(u);
+    /*@-refcounttrans@*/ FREE(u); /*@-refcounttrans@*/
+    return NULL;
 }
 
-urlinfo *newUrlinfo(void)
+void urlFreeCache(void)
 {
-    urlinfo *u;
-    if ((u = malloc(sizeof(*u))) == NULL)
-	return NULL;
-    memset(u, 0, sizeof(*u));
-    u->proxyp = -1;
-    u->port = -1;
-    u->ftpControl = -1;
-    u->ftpGetFileDoneNeeded = 0;
-    return u;
+    int i;
+    for (i = 0; i < uCount; i++) {
+	if (uCache[i] == NULL) continue;
+	uCache[i] = urlFree(uCache[i], "uCache");
+	if (uCache[i])
+	    fprintf(stderr, _("warning: uCache[%d] %p nrefs(%d) != 1 (%s %s)\n"),
+		i, uCache[i], uCache[i]->nrefs,
+		uCache[i]->host, uCache[i]->service);
+    }
+    if (uCache)
+	free(uCache);
+    uCache = NULL;
+    uCount = 0;
 }
 
 static int urlStrcmp(const char *str1, const char *str2)
 {
-    if (str1 && str2) {
+    if (str1 && str2)
 	return (strcmp(str1, str2));
-    } else
-	if (str1 != str2)
-	    return -1;
+    if (str1 != str2)
+	return -1;
     return 0;
 }
 
-static void findUrlinfo(urlinfo **uret, int mustAsk)
+static void urlFind(urlinfo *uret, int mustAsk)
 {
-    urlinfo *u;
-    urlinfo **empty;
-    /*@only@*/static urlinfo **uCache = NULL;
-    static int uCount = 0;
+    urlinfo u;
+    int ucx;
     int i;
 
     if (uret == NULL)
 	return;
 
     u = *uret;
+    URLSANE(u);
 
-    empty = NULL;
+    ucx = -1;
     for (i = 0; i < uCount; i++) {
-	urlinfo *ou;
+	urlinfo ou;
 	if ((ou = uCache[i]) == NULL) {
-	    if (empty == NULL)
-		empty = &uCache[i];
+	    if (ucx < 0)
+		ucx = i;
 	    continue;
 	}
+
 	/* Check for cache-miss condition. A cache miss is
 	 *    a) both items are not NULL and don't compare.
 	 *    b) either of the items is not NULL.
 	 */
 	if (urlStrcmp(u->service, ou->service))
 	    continue;
-	if (urlStrcmp(u->service, ou->service))
-	    continue;
     	if (urlStrcmp(u->host, ou->host))
 	    continue;
 	if (urlStrcmp(u->user, ou->user))
 	    continue;
-	if (urlStrcmp(u->password, ou->password))
-	    continue;
 	if (urlStrcmp(u->portstr, ou->portstr))
 	    continue;
 	break;	/* Found item in cache */
     }
 
     if (i == uCount) {
-	if (empty == NULL) {
-	    uCount++;
+	if (ucx < 0) {
+	    ucx = uCount++;
 	    if (uCache)
 		uCache = xrealloc(uCache, sizeof(*uCache) * uCount);
 	    else
 		uCache = xmalloc(sizeof(*uCache));
-	    empty = &uCache[i];
 	}
-	*empty = u;
+	uCache[ucx] = urlLink(u, "uCache (miss)");
+	u = urlFree(u, "urlSplit (urlFind miss)");
     } else {
-	/* Swap original url and path into the cached structure */
-	const char *up = uCache[i]->path;
-	uCache[i]->path = u->path;
-	u->path = up;
-	up = uCache[i]->url;
-	uCache[i]->url = u->url;
-	u->url = up;
-	freeUrlinfo(u);
+	ucx = i;
+	u = urlFree(u, "urlSplit (urlFind hit)");
     }
 
     /* This URL is now cached. */
-    *uret = u = uCache[i];
+
+    u = urlLink(uCache[ucx], "uCache");
+    *uret = u;
+    u = urlFree(u, "uCache (urlFind)");
 
     /* Zap proxy host and port in case they have been reset */
     u->proxyp = -1;
     FREE(u->proxyh);
 
     /* Perform one-time FTP initialization */
-    if (u->service && !strcmp(u->service, "ftp")) {
+    if (u->urltype == URL_IS_FTP) {
 
 	if (mustAsk || (u->user != NULL && u->password == NULL)) {
 	    char * prompt;
-	    FREE(u->password);
-	    prompt = alloca(strlen(u->host) + strlen(u->user) + 40);
+	    prompt = alloca(strlen(u->host) + strlen(u->user) + 256);
 	    sprintf(prompt, _("Password for %s@%s: "), u->user, u->host);
-	    u->password = xstrdup(getpass(prompt));
+	    FREE(u->password);
+	    u->password = xstrdup( /*@-unrecog@*/ getpass(prompt) /*@=unrecog@*/ );
 	}
 
 	if (u->proxyh == NULL) {
@@ -176,7 +251,7 @@ static void findUrlinfo(urlinfo **uret, int mustAsk)
     }
 
     /* Perform one-time HTTP initialization */
-    if (u->service && !strcmp(u->service, "http")) {
+    if (u->urltype == URL_IS_HTTP) {
 
 	if (u->proxyh == NULL) {
 	    const char *proxy = rpmExpand("%{_httpproxy}", NULL);
@@ -205,39 +280,91 @@ static void findUrlinfo(urlinfo **uret, int mustAsk)
     return;
 }
 
+static struct urlstring {
+    const char *leadin;
+    urltype	ret;
+} urlstrings[] = {
+    { "file://",	URL_IS_PATH },
+    { "ftp://",		URL_IS_FTP },
+    { "http://",	URL_IS_HTTP },
+    { "-",		URL_IS_DASH },
+    { NULL,		URL_IS_UNKNOWN }
+};
+
+urltype urlIsURL(const char * url) {
+    struct urlstring *us;
+
+    if (url && *url) {
+	for (us = urlstrings; us->leadin != NULL; us++) {
+	    if (strncmp(url, us->leadin, strlen(us->leadin)))
+		continue;
+	    return us->ret;
+	}
+    }
+
+    return URL_IS_UNKNOWN;
+}
+
+/* Return path portion of url (or pointer to NUL if url == NULL) */
+int urlPath(const char * url, const char ** pathp)
+{
+    const char *path;
+    int urltype;
+
+    path = url;
+    urltype = urlIsURL(url);
+    switch (urltype) {
+    case URL_IS_FTP:
+	url += sizeof("ftp://") - 1;
+	path = strchr(url, '/');
+	if (path == NULL) path = url + strlen(url);
+	break;
+    case URL_IS_HTTP:
+    case URL_IS_PATH:
+	url += sizeof("file://") - 1;
+	path = strchr(url, '/');
+	if (path == NULL) path = url + strlen(url);
+	break;
+    case URL_IS_UNKNOWN:
+	if (path == NULL) path = "";
+	break;
+    case URL_IS_DASH:
+	path = "";
+	break;
+    }
+    if (pathp)
+	*pathp = path;
+    return urltype;
+}
+
 /*
  * Split URL into components. The URL can look like
  *	service://user:password@host:port/path
  */
-int urlSplit(const char * url, urlinfo **uret)
+int urlSplit(const char * url, urlinfo *uret)
 {
-    urlinfo *u;
+    urlinfo u;
     char *myurl;
     char *s, *se, *f, *fe;
 
     if (uret == NULL)
 	return -1;
-    if ((u = newUrlinfo()) == NULL)
+    if ((u = urlNew("urlSplit")) == NULL)
 	return -1;
 
     if ((se = s = myurl = xstrdup(url)) == NULL) {
-	freeUrlinfo(u);
+	u = urlFree(u, "urlSplit (error #1)");
 	return -1;
     }
 
     u->url = xstrdup(url);
+    u->urltype = urlIsURL(url);
 
-    do {
+    while (1) {
 	/* Point to end of next item */
 	while (*se && *se != '/') se++;
-	if (*se == '\0') {
-	    /* XXX can't find path */
-	    if (myurl) free(myurl);
-	    freeUrlinfo(u);
-	    return -1;
-	}
 	/* Item was service. Save service and go for the rest ...*/
-    	if ((se != s) && se[-1] == ':' && se[0] == '/' && se[1] == '/') {
+    	if (*se && (se != s) && se[-1] == ':' && se[0] == '/' && se[1] == '/') {
 		se[-1] = '\0';
 	    u->service = xstrdup(s);
 	    se += 2;	/* skip over "//" */
@@ -245,11 +372,10 @@ int urlSplit(const char * url, urlinfo **uret)
 	    continue;
 	}
 	
-	/* Item was everything-but-path. Save path and continue parse on rest */
-	u->path = xstrdup(se);
+	/* Item was everything-but-path. Continue parse on rest */
 	*se = '\0';
 	break;
-    } while (1);
+    }
 
     /* Look for ...@host... */
     fe = f = s;
@@ -278,7 +404,7 @@ int urlSplit(const char * url, urlinfo **uret)
 	    if (!(end && *end == '\0')) {
 		rpmMessage(RPMMESS_ERROR, _("url port must be a number\n"));
 		if (myurl) free(myurl);
-		freeUrlinfo(u);
+		u = urlFree(u, "urlSplit (error #3)");
 		return -1;
 	    }
 	}
@@ -290,207 +416,77 @@ int urlSplit(const char * url, urlinfo **uret)
 	serv = /*@-unrecog@*/ getservbyname(u->service, "tcp") /*@=unrecog@*/;
 	if (serv != NULL)
 	    u->port = ntohs(serv->s_port);
-	else if (!strcasecmp(u->service, "ftp"))
+	else if (u->urltype == URL_IS_FTP)
 	    u->port = IPPORT_FTP;
-	else if (!strcasecmp(u->service, "http"))
+	else if (u->urltype == URL_IS_HTTP)
 	    u->port = IPPORT_HTTP;
     }
 
     if (myurl) free(myurl);
     if (uret) {
 	*uret = u;
-	findUrlinfo(uret, 0);
+	urlFind(uret, 0);
     }
     return 0;
 }
 
-static int urlConnect(const char * url, /*@out@*/urlinfo ** uret)
-{
-    urlinfo *u;
-
-    if (urlSplit(url, &u) < 0)
-	return -1;
-
-    if (!strcmp(u->service, "ftp") && u->ftpControl < 0) {
-
-	u->ftpGetFileDoneNeeded = 0;	/* XXX PARANOIA */
-	rpmMessage(RPMMESS_DEBUG, _("logging into %s as %s, pw %s\n"),
-		u->host,
-		u->user ? u->user : "ftp",
-		u->password ? u->password : "(username)");
-
-	u->ftpControl = ftpOpen(u);
-
- 	if (u->ftpControl < 0) {	/* XXX save ftpOpen error */
-	    u->openError = u->ftpControl;
-	    return u->ftpControl;
-	}
-
-    }
-
-    if (uret != NULL)
-	*uret = u;
-
-    return 0;
-}
-
-urltype urlIsURL(const char * url)
-{
-    struct urlstring *us;
-
-    for (us = urlstrings; us->leadin != NULL; us++) {
-	if (strncmp(url, us->leadin, strlen(us->leadin)))
-	    continue;
-	return us->ret;
-    }
-
-    return URL_IS_UNKNOWN;
-}
-
-#ifdef	NOTYET
-int urlAbort(FD_t fd)
-{
-    if (fd != NULL && fd->fd_url) {
-	urlinfo *u = (urlinfo *)fd->fd_url;
-	if (u->ftpControl >= 0)
-	    ftpAbort(fd);
-    }
-}
-#endif
-
-int ufdClose(FD_t fd)
-{
-    if (fd != NULL && fd->fd_url) {
-	urlinfo *u = (urlinfo *)fd->fd_url;
-	/* Close the ftp control channel (not strictly necessary, but ... */
-	if (u->ftpControl >= 0) {
-	    ftpAbort(fd);
-	    fd = NULL;	/* XXX ftpAbort does fdClose(fd) */
-	    close(u->ftpControl);
-	    u->ftpControl = -1;
-	}
-    }
-    return fdClose(fd);
-}
-
-FD_t ufdOpen(const char *url, int flags, mode_t mode)
-{
-    FD_t fd = NULL;
-    urlinfo *u;
-
-    switch (urlIsURL(url)) {
-    case URL_IS_FTP:
-	if (urlConnect(url, &u) < 0)
-	    break;
-	if ((fd = fdNew()) == NULL)
-	    break;
-	fd->fd_url = u;
-	if ((u->openError = ftpGetFileDesc(fd)) < 0) {
-	    u->ftpControl = -1;
-	    fd = NULL;	/* XXX fd already closed */
-	}
-	break;
-    case URL_IS_HTTP:
-	if (urlSplit(url, &u))
-	    break;
-	if ((fd = fdNew()) == NULL)
-	    break;
-	fd->fd_url = u;
-	fd->fd_fd = httpOpen(u);
-	if (fd->fd_fd < 0)		/* XXX Save httpOpen error */
-	    u->openError = fd->fd_fd;
-	break;
-    case URL_IS_PATH:
-	if (urlSplit(url, &u))
-	    break;
-	fd = fdOpen(u->path, flags, mode);
-	break;
-    case URL_IS_DASH:
-	fd = fdDup(STDIN_FILENO);
-	break;
-    case URL_IS_UNKNOWN:
-    default:
-	fd = fdOpen(url, flags, mode);
-	break;
-    }
-
-    if (fd == NULL || fdFileno(fd) < 0) {
-	ufdClose(fd);
-	return NULL;
-    }
-    return fd;
-}
-
 int urlGetFile(const char * url, const char * dest) {
     int rc;
     FD_t sfd = NULL;
     FD_t tfd = NULL;
+    const char * sfuPath = NULL;
+    int urlType = urlPath(url, &sfuPath);
 
-    sfd = ufdOpen(url, O_RDONLY, 0);
-    if (sfd == NULL || fdFileno(sfd) < 0) {
-	rpmMessage(RPMMESS_DEBUG, _("failed to open %s\n"), url);
-	ufdClose(sfd);
+    if (*sfuPath == '\0')
 	return FTPERR_UNKNOWN;
+	
+    sfd = Fopen(url, "r.ufdio");
+    if (sfd == NULL || Ferror(sfd)) {
+	rpmMessage(RPMMESS_DEBUG, _("failed to open %s: %s\n"), url, Fstrerror(sfd));
+	rc = FTPERR_UNKNOWN;
+	goto exit;
     }
 
-    if (sfd->fd_url != NULL && dest == NULL) {
-	const char *fileName = ((urlinfo *)sfd->fd_url)->path;
-	if ((dest = strrchr(fileName, '/')) != NULL)
+    if (dest == NULL) {
+	if ((dest = strrchr(sfuPath, '/')) != NULL)
 	    dest++;
 	else
-	    dest = fileName;
+	    dest = sfuPath;
     }
 
-    tfd = fdOpen(dest, O_CREAT|O_WRONLY|O_TRUNC, 0600);
-    if (fdFileno(tfd) < 0) {
-	rpmMessage(RPMMESS_DEBUG, _("failed to create %s\n"), dest);
-	fdClose(tfd);
-	ufdClose(sfd);
-	return FTPERR_UNKNOWN;
+    tfd = Fopen(dest, "w.ufdio");
+if (_url_debug)
+fprintf(stderr, "*** urlGetFile sfd %p %s tfd %p %s\n", sfd, url, tfd, dest);
+    if (tfd == NULL || Ferror(tfd)) {
+	/* XXX Fstrerror */
+	rpmMessage(RPMMESS_DEBUG, _("failed to create %s: %s\n"), dest, Fstrerror(tfd));
+	rc = FTPERR_UNKNOWN;
+	goto exit;
     }
 
-    switch (urlIsURL(url)) {
+    switch (urlType) {
     case URL_IS_FTP:
-	if ((rc = ftpGetFile(sfd, tfd))) {
-	    unlink(dest);
-	    ufdClose(sfd);
-	}
-	/* XXX fdClose(sfd) done by copyData */
-	break;
     case URL_IS_HTTP:
     case URL_IS_PATH:
     case URL_IS_DASH:
-	if ((rc = httpGetFile(sfd, tfd))) {
-	    unlink(dest);
-	    ufdClose(sfd);
-	}
-	/* XXX fdClose(sfd) done by copyData */
-	break;
     case URL_IS_UNKNOWN:
+	if ((rc = ufdGetFile(sfd, tfd))) {
+	    Unlink(dest);
+	    /* XXX FIXME: sfd possibly closed by copyData */
+	    /*@-usereleased@*/ Fclose(sfd) /*@=usereleased@*/ ;
+	}
+	sfd = NULL;	/* XXX Fclose(sfd) done by ufdGetFile */
+	break;
     default:
 	rc = FTPERR_UNKNOWN;
 	break;
     }
 
-    fdClose(tfd);
+exit:
+    if (tfd)
+	Fclose(tfd);
+    if (sfd)
+	Fclose(sfd);
 
     return rc;
 }
-
-const char *urlStrerror(const char *url)
-{
-    const char *retstr;
-    urlinfo *u;
-    switch (urlIsURL(url)) {
-/* XXX This only works for httpOpen/ftpOpen/ftpGetFileDesc failures */
-    case URL_IS_FTP:
-    case URL_IS_HTTP:
-	retstr = !urlSplit(url, &u)
-		? ftpStrerror(u->openError) : "Malformed URL";
-	break;
-    default:
-	retstr = strerror(errno);
-	break;
-    }
-    return retstr;
-}
