@@ -1,6 +1,6 @@
 #include "system.h"
 
-#include "rpmlib.h"
+#include <rpmlib.h>
 
 int main(int argc, char ** argv)
 {
@@ -9,22 +9,22 @@ int main(int argc, char ** argv)
 
     setprogname(argv[0]);	/* Retrofit glibc __progname */
     if (argc == 1) {
-	fdi = fdDup(STDIN_FILENO);
+	fdi = Fopen("-", "r.ufdio");
     } else {
-	fdi = fdOpen(argv[1], O_RDONLY, 0644);
+	fdi = Fopen(argv[1], "r.ufdio");
     }
 
-    if (fdFileno(fdi) < 0) {
+    if (fdi == NULL || Ferror(fdi)) {
 	fprintf(stderr, _("cannot open %s: %s\n"), argv[1], strerror(errno));
-	exit(1);
+	exit(EXIT_FAILURE);
     }
 
     h = headerRead(fdi, HEADER_MAGIC_YES);
     if (!h) {
 	fprintf(stderr, _("headerRead error: %s\n"), strerror(errno));
-	exit(1);
+	exit(EXIT_FAILURE);
     }
-    fdClose(fdi);
+    Fclose(fdi);
   
     headerDump(h, stdout, HEADER_DUMP_INLINE, rpmTagTable);
     headerFree(h);
